import { createStore } from 'vuex'
import { v4 as uuidv4 } from "uuid"
import { instance } from "./axios"


export default createStore({
  state: {
    projects: [],
    files: [],
    clips: [],
    user: null,
    errors: [],
    previewFile: null,
    uploads: []
  },

  mutations: {
    addError(state, message) {
      if (typeof(message) == "object") {
        // Loop through object keys (potentially multiple errors)
        for (const key in message) {
          let error = {'id': uuidv4(), 'message': `${key}: ${message[key]}`}
          state.errors.push(error)
        }
      } else {
        // Single error message
        let error = {'id': uuidv4(), 'message': message}
        state.errors.push(error)
      }
    },
    removeError(state, error) {
      state.errors = state.errors.filter(err => err.id != error.id )
    },
    clearErrors(state) {
      state.errors = []
    },
    setUser(state, user) {
      state.user = user
    },
    setProjects(state, projects) {
      state.projects = projects
    },
    setFiles(state, files) {
      state.files = files
    },
    setClips(state, clips) {
      state.clips = clips
    },
    deleteClip(state, clip_id) {
      state.clips = state.clips.filter(clip => clip.id != clip_id)
    },
    setPreviewFile(state, file) {
      state.previewFile = file
    },
    addUpload(state, upload) {
      state.uploads.push(upload)
    },
    updateUpload(state, upload) {
      for (const up of state.uploads) {
        if (up.id == upload.id) {
          up.progress = upload.progress
          break
        }
      }
    },
    removeUpload(state, uploadID) {
      state.uploads = state.uploads.filter(up => up.id != uploadID )
    },
  },
  getters: {
    isAuthenticated: state => !!state.user
  },
  actions: {
    async login({commit}, auth) {
      try {
        const response = await instance.get('/users/')
        let user_object = response.data.results[0]
        user_object.auth = auth
        commit('setUser', user_object)
      } catch(err) {
        commit('setUser', null)
        commit('addError', err.response.data)
      }
    },
    async loadProjects({commit}) {
      try {
        const response = await instance.get('/projects/')
        commit('setProjects', response.data.results)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async getProject({commit}, project_id) {
      try {
        return instance.get(`projects/${project_id}/`)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async createProject({dispatch, commit}, payload) {
      try {
        await instance.post('/projects/', payload)
        dispatch('loadProjects')
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async deleteProject({commit, dispatch}, project_id) {
      try {
        await instance.delete(`projects/${project_id}/`)
        dispatch('loadProjects')
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async loadFiles({commit}, project_id) {
      try {
        const response = await instance.get(`projects/${project_id}/files/`)
        let files = response.data.results

        // Generate thumbnail for each file object
        for (let fileObj of files) {
          let data = {
            "frame_number": 1,
            "width": 480,
            "height": 270,
            "image_format": "JPEG",
            "image_quality": 100
          }
          try {
            const response = await instance.post(`${fileObj.url}thumbnail/`, data, { responseType: 'arraybuffer' })
            let blob = new Blob(
                [response.data],
                { type: response.headers['content-type'] }
            )
            // Append thumbnail attribute to each file object
            let thumbnailUrl = URL.createObjectURL(blob)
            fileObj.thumbnail = thumbnailUrl

          } catch(err) {
            commit('addError', err.response.data)
          }
        }
        commit('setFiles', files)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async loadClips({commit}, project_id) {
      try {
        const response = await instance.get(`projects/${project_id}/clips/`)
        let clips = response.data.results

        for (let clipObj of clips) {
          // Get related file object
          const file_response = await instance.get(clipObj.file)
          clipObj.fileObj = file_response.data

          let fps = clipObj.json.reader.fps.num / clipObj.json.reader.fps.den
          let data = {
            "frame_number": clipObj.start * fps,
            "width": 480,
            "height": 270,
            "image_format": "JPEG",
            "image_quality": 100
          }
          try {
            const response = await instance.post(`${clipObj.file}thumbnail/`, data, { responseType: 'arraybuffer' })
            let blob = new Blob(
                [response.data],
                { type: response.headers['content-type'] }
            )
            // Append thumbnail attribute to each clip object
            let thumbnailUrl = URL.createObjectURL(blob)
            clipObj.thumbnail = thumbnailUrl

          } catch(err) {
            commit('addError', err.response.data)
          }
        }
        commit('setClips', clips)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async createClip({dispatch, commit}, payload) {
      try {
        await instance.post(`${payload.project_url}clips/`, payload.data)
        dispatch('loadClips', payload.project_id)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async deleteClip({commit}, clip_id) {
      try {
        await instance.delete(`clips/${clip_id}/`)
        commit('deleteClip', clip_id)
      } catch(err) {
        commit('addError', err.response.data)
      }
    },
    async createFile({dispatch, commit}, payload) {
      let uploadID = uuidv4()
      let uploadObject = { 'id': uploadID, progress: 0 }
      commit('addUpload', uploadObject)

      const config = {
        onUploadProgress: progressEvent => {
          let progress = (progressEvent.loaded / progressEvent.total) * 100;
          let uploadObject = { 'id': uploadID, progress }
          commit('updateUpload', uploadObject)
        }
      }

      try {
        await instance.post(`${payload.project_url}files/`, payload.data, config)
        commit('removeUpload', uploadID)
        dispatch('loadFiles', payload.project_id)
      } catch(err) {
        commit('removeUpload', uploadID)
        commit('addError', err.response.data)
      }
    }
  },
  modules: {
  }
})
